/*************************************************************
Module name: DllWork.c
Written by:  Jeffrey Richter
Purpose:     A DLL that creates multiple threads when loaded
*************************************************************/


#define STRICT
#include <Windows.h>
#define DLLWORKAPI __declspec(dllexport)
#include "DllWork.h"


//////////////////////////////////////////////////////////////


// The handle of this DLL Module
HINSTANCE g_hinstDll;

// Event used to signal when threads should terminate
HANDLE g_hEventTerminate;


//////////////////////////////////////////////////////////////


DWORD WINAPI ThreadFunc (LPVOID lp) {
   WaitForSingleObject(g_hEventTerminate, INFINITE);
   DebugBox("Worker thread is terminating", "DllWork");
   FreeLibraryAndExitThread(g_hinstDll, 0);
   return(0);  // Never executes but makes the compiler happy
}


//////////////////////////////////////////////////////////////


void ShutdownLibrary() {
   // Notify the worker threads to shutdown
   SetEvent(g_hEventTerminate);
}


//////////////////////////////////////////////////////////////


void SomeFunc() {
   MessageBox(NULL, "Doing something", 
      "SomeFunc in DllWork", MB_OK);
}


//////////////////////////////////////////////////////////////


void IncrementLibraryUsageCount(HINSTANCE hinst, int nCount) {
   TCHAR szModuleName[_MAX_PATH];
   GetModuleFileName(hinst, szModuleName, _MAX_PATH);
   while (nCount--) LoadLibrary(szModuleName);
}


//////////////////////////////////////////////////////////////


BOOL WINAPI DllMain (HINSTANCE hinstDll, 
   DWORD fdwReason, LPVOID fImpLoad) {

   int nThread;
   const int nMaxWorkerThreads = 2;

   switch (fdwReason) {
   case DLL_PROCESS_DETACH:
      DebugBox("Dll unloading", "DllWork");
      break;

   case DLL_THREAD_ATTACH:
   case DLL_THREAD_DETACH:
      break;

   case DLL_PROCESS_ATTACH:
      DebugBox("Dll loading", "DllWork");

      // Save this module's handle
      g_hinstDll = hinstDll;

      // When signalled, this event tells the 
      // worker threads when to terminte
      g_hEventTerminate = CreateEvent(NULL, TRUE, FALSE, NULL);

      // Create the worker threads
      nThread = 0;
      for (; nThread < nMaxWorkerThreads; nThread++) {
         DWORD dwThreadId;
         CloseHandle(CreateThread(NULL, 0, 
            ThreadFunc, 0, 0, &dwThreadId));
      }

      // Increment this module's usage count so that it is
      // not freed while threads are still executing its code
      IncrementLibraryUsageCount(hinstDll, 
         nMaxWorkerThreads - 1);
      break;
   }

   return(TRUE);  // Successful initialization
}


///////////////////////// End of File ////////////////////////
