/*************************************************************
Module name: UnloadDll.c
Written by:  Jeffrey Richter
Purpose:     EXE source code to dynamically load a DLL
*************************************************************/


#define STRICT
#include <Windows.h>


//////////////////////////////////////////////////////////////


// Data type for explicitly linked function
typedef void (WINAPI* PFNSOMEFUNC)(void);


//////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe, 
   HINSTANCE hinstExePrev, LPSTR pszCmdLine, int nCmdShow) {

   // Explicitly load the DllStub module. Since this module
   // has function forwarders to DllWork, the loader will 
   // also load the DllWork module.
   HINSTANCE hDll = LoadLibrary("DllStub");
   PFNSOMEFUNC pfnSomeFunc;

   if (hDll == NULL) {
      MessageBox(NULL, "Can't load DllStub.DLL", 
         "UnloadDll", MB_OK);
      return(0);
   }

   // Get the address of the function.  When we request
   // the address of the function from DllStub 
   // GetProcAddress sees that the function is forwarded 
   // and returns the address of the function inside DllWork
   pfnSomeFunc = (PFNSOMEFUNC) 
      GetProcAddress(hDll, "SomeFunc");

   // Call the function that is in the DllWork module
   pfnSomeFunc();

   // Freeing DllStub causes it to get a DLL_PROCESS_DETACH
   // notification.  This calls DllWork's ShutDownLibrary 
   // function which terminates the threads in DllWork.
   // Then both DllStub and DllWork are unloaded
   FreeLibrary(hDll);

   MessageBox(NULL, "Ready to terminate process", 
      "UnloadDll", MB_OK);

   return(0);
}


///////////////////////// End of File ////////////////////////
