////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\adapter.h_v   1.0   24 Jun 1996 16:24:28   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:28  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	adapter.h
// Author:		Jeff Huckins
// Abstract:	CAdapterObject Interface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef ADAPTER_H
#define ADAPTER_H

#include "linteger.h"

class CAdapterObject
{
public:
	// ctor/dtor
	CAdapterObject(
		IN ULONG CommonBufferLength = 0,
		IN ULONG CommonBufferAlignment = 0,
		IN BOOLEAN bCacheEnabled = FALSE );
	virtual ~CAdapterObject();

	// initialization routines
	NTSTATUS Initialize( 
		IN PUNICODE_STRING RegistryPath, 
		IN ULONG CommonBufferLength,
		IN ULONG CommonBufferLengthAlignment,
		IN BOOLEAN bCacheEnabled,
		IN PDEVICE_DESCRIPTION DeviceDescription );
	NTSTATUS Initialize(
		IN PUNICODE_STRING RegistryPath, 
		IN PDEVICE_DESCRIPTION DeviceDescription );

	// cleanup routine
	VOID Unload( void );

	// Calls HalGetAdapter()
	BOOLEAN GetAdapter( IN PDEVICE_DESCRIPTION DeviceDescription );

	// Calls HalAllocateCommonBuffer()
	BOOLEAN AllocateCommonBuffer( void );

	// Calls HalFreeCommonBuffer()
	VOID FreeCommonBuffer( void );

	// Get/Set members
	ULONG GetNumberOfMapRegisters( void ) const;
	PVOID GetCommonBuffer( void ) const;
	ULONG GetCommonBufferLength( void ) const;
	const PHYSICAL_ADDRESS& GetCommonBufferLogicalAddress( void ) const;

	// overridables
	virtual NTSTATUS OnInitialize( IN PUNICODE_STRING RegistryPath );
	virtual VOID OnUnload( void );

private:
	// members
	PADAPTER_OBJECT	m_AdapterObject;
	ULONG			m_NumberOfMapRegisters;
	PVOID			m_CommonBuffer;
	PVOID			m_CommonBufferAligned;
	ULONG			m_CommonBufferLength;
	ULONG			m_CommonBufferAlignment;
	CLargeInteger	m_CommonBufferLogicalAddress;
	CLargeInteger	m_CommonBufferLogicalAddressAligned;
	BOOLEAN			m_bCacheEnabled;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline ULONG CAdapterObject::GetNumberOfMapRegisters( void ) const
{
	return m_NumberOfMapRegisters;
}

inline PVOID CAdapterObject::GetCommonBuffer( void ) const
{
	return m_CommonBufferAligned;
}

inline ULONG CAdapterObject::GetCommonBufferLength( void ) const
{
	return m_CommonBufferLength;
}

inline const PHYSICAL_ADDRESS& CAdapterObject::GetCommonBufferLogicalAddress( void ) const
{
	return (const LARGE_INTEGER) m_CommonBufferLogicalAddressAligned;
}

#endif

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\adapter.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:28   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.7   03 Jun 1996 07:47:00   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.6   31 May 1996 15:49:26   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.5   13 May 1996 13:26:36   JDELLIOT
 * added common buffer alignment support
 * 
 *    Rev 1.4   04 Apr 1996 09:45:20   JHUCKINS
 * Added IN and OUT macros to formal argument lists
 * 
 *    Rev 1.3   27 Mar 1996 14:58:28   JDELLIOT
 * 
 *    Rev 1.2   27 Mar 1996 14:12:32   JDELLIOT
 * added PVCS keywords and support for warning level 4
 *
 */
