////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\device.cpv   1.0   24 Jun 1996 16:24:24   JHUCKINS  $
//
// INTEL Corporation Proprietary Information
//
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
//
// Copyright (c) 1996 Intel Corporation. All rights reserved.
//
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:24  $
// $Author:   JHUCKINS  $
//
// Log at end of file.
//
// Module Name:		device.cpp
// Abstract:		CDeviceObject Implementation
// Environment:		Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
//
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4355 )	// 'this' : used in base member initializer list
#pragma warning( disable: 4705 )	// Statement has no effect
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <idrivrpp.h>
#include "driver.h"
#include "device.h"
#include "debug.h"

////////////////////////////////////////////////////////////////////////////////
// ctor/dtor
//
CDeviceObject::CDeviceObject( IN CDriverObject* DriverObject /* = NULL */ )
	:	m_DriverObject( DriverObject )
{
	// initialize structures (do not initialize m_Win32Device - it is done in new)
	m_RegistryPath.Buffer = NULL;
	m_RegistryPath.Length = 0;
	m_RegistryPath.MaximumLength = 0;
}

CDeviceObject::~CDeviceObject()
{
}

////////////////////////////////////////////////////////////////////////////////
// storage management
//
PVOID CDeviceObject::operator new( size_t /*size*/ )
{
	return NULL;
}

PVOID CDeviceObject::operator new(
	IN size_t size, 
	IN PDRIVER_OBJECT DriverObject, 
	IN PUNICODE_STRING DeviceName, 
	IN PUNICODE_STRING Win32DeviceName, 
	IN DEVICE_TYPE DeviceType, 
	IN ULONG DeviceCharacteristics, 
	IN BOOLEAN Exclusive, 
	OUT NTSTATUS& status )
{
	ASSERT( NULL != DriverObject );
	ASSERT( NULL != DeviceName );
	ASSERT( NULL != Win32DeviceName );

	PDEVICE_OBJECT DeviceObject = NULL;
 	CDeviceObject* pThis = NULL;

	// create the device, allocating space for this object
	if( STATUS_SUCCESS ==
		(status = IoCreateDevice(
			DriverObject, 
			size, 
			DeviceName, 
			DeviceType, 
			DeviceCharacteristics, 
			Exclusive, 
			&DeviceObject )) )
	{
		ASSERT( NULL != DeviceObject );
		pThis = (CDeviceObject*) DeviceObject->DeviceExtension;

		// The device extension area will be this object's image.
		// Set up the data member pointing to the DEVICE_OBJECT allocated by the IOManager.
		ASSERT( NULL != pThis );
		pThis->m_DeviceObject = DeviceObject;

		// @@@ do we need to force the client to create this symbolic link?
	    if( STATUS_SUCCESS == (status = IoCreateSymbolicLink( Win32DeviceName, DeviceName )) )
		{
			// allocate space for m_Win32DeviceName
			// @@@ can we use CUnicodeString here
			pThis->m_Win32DeviceName.Buffer = PWSTR( new( PagedPool ) char[ Win32DeviceName->MaximumLength ] );
			if( NULL != pThis->m_Win32DeviceName.Buffer )
			{
				// set maximum length to allocated length
				pThis->m_Win32DeviceName.MaximumLength = Win32DeviceName->MaximumLength;

				// copy the string
				RtlCopyUnicodeString( &pThis->m_Win32DeviceName, Win32DeviceName );
			}
			else
			{
				status = STATUS_NO_MEMORY;
				TRACE( "CDeviceObject::operator new - Failed to allocate space for m_Win32DeviceName!" );
				// cleanup occurs in operator delete()
			}
		}
		else
		{
			TRACE( "CDeviceObject::operator new - Failed on call to IoCreateSymbolicLink!" );
			// cleanup occurs in operator delete()
		}
	}
	else
	{
		TRACE( "CDeviceObject::operator new - Failed on call to IoCreateDevice, returning NULL!" );
	}

	return pThis;
}

VOID CDeviceObject::operator delete( IN PVOID p, IN size_t /*size*/ )
{
	ASSERT( NULL != p );
	CDeviceObject* pDevice = (CDeviceObject*) p;

	// did we assign the device object member
	if( NULL != pDevice->m_DeviceObject )
	{
		// did we allocate and assign the m_Win32DeviceNmae member
		if( NULL != pDevice->m_Win32DeviceName.Buffer )
		{
			ASSERT( 0 != pDevice->m_Win32DeviceName.Length );

			// delete the symbolic link
			IoDeleteSymbolicLink( &pDevice->m_Win32DeviceName );
			delete pDevice->m_Win32DeviceName.Buffer;

			// reset members
			pDevice->m_Win32DeviceName.Buffer = NULL;
			pDevice->m_Win32DeviceName.Length = 0;
			pDevice->m_Win32DeviceName.MaximumLength = 0;
		}

		// Free up the DEVICE_OBJECT
		IoDeleteDevice( pDevice->m_DeviceObject );
	}

	return;
}

CDeviceObject::operator PDEVICE_OBJECT()
{
	return m_DeviceObject;
}

NTSTATUS CDeviceObject::Initialize( PUNICODE_STRING RegistryPath )
{
	// allocate space for m_RegistryPath
	// @@@ can we use CUnicodeString here?
	// @@@ do we really need to allocate MaximumLength... why not Length + 1, also
	// @@@ aren't MaximumLength and Length in bytes not chars (i.e. new(...) char[ size ])
	m_RegistryPath.Buffer = new( PagedPool ) WCHAR[ RegistryPath->MaximumLength ];
	if( NULL == m_RegistryPath.Buffer )
	{
		TRACE( "CDeviceObject::Initialize - Failed to allocate buffer for RegistryPath!" );
		return STATUS_NO_MEMORY;
	}

	m_RegistryPath.MaximumLength = RegistryPath->MaximumLength;
	RtlCopyUnicodeString( &m_RegistryPath, RegistryPath );

	return OnInitialize( RegistryPath );
}

NTSTATUS CDeviceObject::OnInitialize( PUNICODE_STRING /* RegistryPath */ )
{
	return STATUS_SUCCESS;
}

// This object should not be deleted by CDriverObject until after this call
VOID CDeviceObject::Unload()
{
	OnUnload();

	// did we allocate and assign the m_RegistryPath member
	if( NULL != m_RegistryPath.Buffer )
	{
		// free up the memory for the registry path buffer.
		delete m_RegistryPath.Buffer;

		// reset members
		m_RegistryPath.Buffer = NULL;
		m_RegistryPath.Length = 0;
		m_RegistryPath.MaximumLength = 0;
	}

	return;
}

VOID CDeviceObject::OnUnload( void )
{
	return;
}

BOOLEAN CDeviceObject::OnSynchronizeExecutionForRead( void )
{
	return TRUE;
}

BOOLEAN CDeviceObject::OnSynchronizeExecutionForWrite( void )
{
	return TRUE;
}

VOID CDeviceObject::OnStartIo( IN PIRP Irp )
{
	ASSERT_IRQL_EQ( DISPATCH_LEVEL );

	// Complete the request
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
}

VOID CDeviceObject::OnStartReadIo( IN PIRP Irp )
{
	ASSERT_IRQL_EQ( DISPATCH_LEVEL );

	PIO_STACK_LOCATION CurrentStackLocation = IoGetCurrentIrpStackLocation( Irp );

	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = CurrentStackLocation->Parameters.Read.Length;
	// @@@ do we want to set Information to 0 and call IoCompleteRequest
}

VOID CDeviceObject::OnStartWriteIo( IN PIRP Irp )
{
	ASSERT_IRQL_EQ( DISPATCH_LEVEL );

	PIO_STACK_LOCATION CurrentStackLocation = IoGetCurrentIrpStackLocation( Irp );

	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = CurrentStackLocation->Parameters.Write.Length;
	// @@@ do we want to set Information to 0 and call IoCompleteRequest
}

////////////////////////////////////////////////////////////////////////////////
// overridable dispatch handlers
//

NTSTATUS CDeviceObject::OnIrpMjCreate( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjCreate Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjCreateNamedPipe( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjCreateNamedPipe Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjClose( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjClose Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjRead( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjRead Entered" );

	// Mark the request pending
	IoMarkIrpPending( Irp );
	StartPacket( Irp, NULL );

	return STATUS_PENDING;
}

NTSTATUS CDeviceObject::OnIrpMjWrite( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjWrite Entered" );

	// Mark the request pending
	IoMarkIrpPending( Irp );
	StartPacket( Irp, NULL );

	return STATUS_PENDING;
}

NTSTATUS CDeviceObject::OnIrpMjQueryInformation( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjQueryInformation Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjSetInformation( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjSetInformation Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjQueryEa( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjQueryEa Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjSetEa( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjSetEa Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjFlushBuffers( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjFlushBuffers Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjQueryVolumeInformation( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjQueryVolumeInformation Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjSetVolumeInformation( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjSetVolumeInformation Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjDirectoryControl( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjDirectoryControl Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjFileSystemControl( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjFileSystemControl Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjDeviceControl( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjDeviceControl Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjInternalDeviceControl( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjInternalDeviceControl Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjShutdown( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjShutdown Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjLockControl( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjLockControl Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjCleanup( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjCleanup Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjCreateMailslot( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjCreateMailslot Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjQuerySecurity( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjQuerySecurity Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjSetSecurity( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjSetSecurity Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjQueryPower( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjQueryPower Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjSetPower( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjSetPower Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjDeviceChange( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjDeviceChange Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjQueryQuota( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjQueryQuota Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjSetQuota( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjSetQuota Entered" );
	return OnIrpMjDefault( Irp );
}

NTSTATUS CDeviceObject::OnIrpMjDefault( IN PIRP Irp )
{
	TRACE( "CDeviceObject::OnIrpMjDefault Entered" );

	// Complete the request
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );

	return STATUS_SUCCESS;
}

BOOLEAN CDeviceObject::OnInterrupt( VOID )
{
	return FALSE;
}

VOID CDeviceObject::OnDpc( IN PIRP /* Irp */, IN PVOID /* Context */ )
{
	return;
}

BOOLEAN CDeviceObject::CancelIrp( IN PIRP Irp )
{
	KIRQL Irql;
	IoAcquireCancelSpinLock( &Irql );

	BOOLEAN bResult = IoCancelIrp( Irp );

	IoReleaseCancelSpinLock( Irql );

	return bResult;
}

VOID CDeviceObject::Cancel( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp )
{
	CDeviceObject* pDevice = (CDeviceObject*) DeviceObject->DeviceExtension;
	ASSERT( NULL != pDevice );

	pDevice->OnCancel( Irp );
	return;
}

VOID CDeviceObject::OnCancel( IN PIRP Irp )
{
	Irp->IoStatus.Status = STATUS_CANCELLED;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return;
}

BOOLEAN CDeviceObject::PciFindDevice(
	IN USHORT VendorId, 
	IN USHORT DeviceId, 
	OUT PULONG BusNumber, 
	OUT PPCI_SLOT_NUMBER PciSlotNumber, 
	OUT PPCI_COMMON_CONFIG PciCommonConfig )
{
	// initialize parameters
	*BusNumber = 0;
	PciSlotNumber->u.AsULONG = 0;

	BOOLEAN bFound = FALSE;
	while( FALSE == bFound )
	{
		// get bus data
		ULONG ulReturn = HalGetBusData( 
			PCIConfiguration, 
			*BusNumber, 
			PciSlotNumber->u.AsULONG, 
			PciCommonConfig, 
			sizeof PCI_COMMON_CONFIG );
		
		// pci bus does not exist (assume that these start at 0 and go up with no gaps)
		if( 0 == ulReturn )
		{
			break;
		}
		else if( sizeof PCI_COMMON_CONFIG == ulReturn )
		{
			// found a card... see if it is ours
			if( VendorId == PciCommonConfig->VendorID && DeviceId == PciCommonConfig->DeviceID )
			{
				// found our card... get its information
				bFound = TRUE;
				break;
			}
		}
		
		//@@@ it appears that only the DeviceNumber of the SlotNumber is used... 
		//@@@ maybe if you have two or more of the same card it is used
		//@@@ if it is never used or if it should be stepped first we could
		//@@@ simplify the code 
		
		// step our BusNumber and/or SlotNumber
		if( 31 == PciSlotNumber->u.bits.DeviceNumber && 7 == PciSlotNumber->u.bits.FunctionNumber )
		{
			// try the next bus
			*BusNumber++;
			PciSlotNumber->u.AsULONG = 0;
		}
		else if( 7 == PciSlotNumber->u.bits.FunctionNumber )
		{
			// try the next device number
			PciSlotNumber->u.bits.DeviceNumber++;
			PciSlotNumber->u.bits.FunctionNumber = 0;
		}
		else
		{
			// try the next function number
			PciSlotNumber->u.bits.FunctionNumber++;
		}
	}

	#ifdef _DEBUG
	//@@@ did we find a device, validate its size
	if( bFound )
	{
		ASSERT( sizeof( PCI_COMMON_CONFIG ) ==
			HalSetBusData(
				PCIConfiguration,
				*BusNumber,
				PciSlotNumber->u.AsULONG,
				PciCommonConfig,
				sizeof PCI_COMMON_CONFIG ) );
	}
	#endif
	
	return bFound;
}

NTSTATUS CDeviceObject::AssignSlotResources(
	IN PUNICODE_STRING RegistryPath, 
	IN PUNICODE_STRING DriverClassName, 
	IN INTERFACE_TYPE BusType,  
	IN ULONG BusNumber, 
	IN ULONG SlotNumber, 
	OUT PCM_RESOURCE_LIST* AllocatedResources )
{
	// If this CDeviceObject is an exclusive device object for this driver, NULL should be passed
	// as the DeviceObject argument to HalAssignSlotResources(), so that device-specific registry
	// management is not required.  If this is not an exclusive CDeviceObject object, then it is
	// necessary to create and update the registry path passed to the DriverEntry routine in order
	// to maintain device-specific resource data for each set of resources claimed under the driver-
	// specific path contained in the DriverEntry's RegistryPath.
	return HalAssignSlotResources( 
		RegistryPath, 
		DriverClassName,
		(PDRIVER_OBJECT) *m_DriverObject,
		m_DeviceObject,
		BusType,
		BusNumber,
		SlotNumber,
		AllocatedResources );
}

NTSTATUS CDeviceObject::AssignResources(
	IN PUNICODE_STRING RegistryPath, 
	IN PUNICODE_STRING DriverClassName, 
	IN PIO_RESOURCE_REQUIREMENTS_LIST RequestedResources, 
	OUT PCM_RESOURCE_LIST* AllocatedResources )
{
	return IoAssignResources( 
		RegistryPath, 
		DriverClassName,
		(PDRIVER_OBJECT) *m_DriverObject,
		m_DeviceObject,
		RequestedResources,
		AllocatedResources );
}

NTSTATUS CDeviceObject::ReportResourceUsage(
	IN PUNICODE_STRING DriverClassName,
	IN PCM_RESOURCE_LIST DeviceList,
	IN ULONG DeviceListSize,
	IN BOOLEAN bOverRideConflict,
	OUT PBOOLEAN bConflictDetected )
{
	return IoReportResourceUsage(
		DriverClassName,
		(PDRIVER_OBJECT) *m_DriverObject,
		NULL,
		0,
		m_DeviceObject,
		DeviceList,
		DeviceListSize,
		bOverRideConflict,
		bConflictDetected );
}

///////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\device.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:24   JHUCKINS
// Initial revision.
// 
//    Rev 1.25   11 Jun 1996 11:31:12   JDELLIOT
// added support for all known Irps
// added default support for uknown Irps
// 
//    Rev 1.24   06 Jun 1996 15:56:54   JHUCKINS
// Added trace output to virtual dispatch routines
// 
//    Rev 1.23   31 May 1996 15:49:44   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
