////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\device.h_v   1.0   24 Jun 1996 16:24:30   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:30  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	device.h
// Abstract:	CDeviceObject Interface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef device_h
#define device_h 

#include "debug.h"

// forward declarations
class CDriverObject;

// CDeviceObject encapsulates the NTDDK DEVICE_OBJECT structure.
class CDeviceObject 
{
public:
	// ctor/dtor
	CDeviceObject( IN CDriverObject* DriverObject = NULL );
	virtual ~CDeviceObject();

	// This operator new must return NULL.
	PVOID operator new( size_t size );

	// Calls IoCreateDevice() passing pointer to stack PDEVICE_OBJECT and size
	// as DeviceExtensionSize. Upon return from IoCreateDevice(),
	// m_DeviceObject data member will be set by casting
	// pDeviceObject->DeviceExtension to CDeviceObject.
	PVOID operator new(
		IN size_t size, 
		IN PDRIVER_OBJECT DriverObject, 
		IN PUNICODE_STRING DeviceName, 
		IN PUNICODE_STRING Win32DeviceName, 
		IN DEVICE_TYPE DeviceType, 
		IN ULONG DeviceCharacteristics, 
		IN BOOLEAN Exclusive, 
		OUT NTSTATUS& status );

	// Should do nothing, since the IOManager will call IoDeleteDevice() when
	// a driver is unloaded.
	VOID operator delete( IN PVOID p, IN size_t size );

	// Cast operator for PDEVICE_OBJECT.
	operator PDEVICE_OBJECT();

	// called from CDriverObject::Initialize. This call is made from the
	// DriverEntry routine running at IRQL_PASSIVE_LEVEL.
	// IMPORTANT:
	//		This routine MUST create the CInterruptObject and CDpcObject
	//		objects contained by value by derived CDeviceObject class to setup
	//		callbacks to OnInterrupt() and OnDpc(), respectively.
	NTSTATUS Initialize( IN PUNICODE_STRING RegistryPath );

	// Nonvirtual called by CDriverDevice::Unload() when the driver is
	// unloading in order to perform cleanup.
	VOID Unload( VOID );

	CDriverObject* GetDriver( VOID );

	// called by CInterruptObject::SynchronizeExecutionForXXX() to
	// synchronize device I/O programming.
	virtual BOOLEAN OnSynchronizeExecutionForRead( VOID );
	virtual BOOLEAN OnSynchronizeExecutionForWrite( VOID );

	// called by CInterruptObject::OnInterrupt()
	virtual BOOLEAN OnInterrupt( VOID );

	// called from CDpcObject::Dpc()
	virtual VOID OnDpc( IN PIRP Irp, IN PVOID Context );

	// overridable IO handlers
	virtual VOID OnStartIo( IN PIRP Irp );
	virtual VOID OnStartReadIo( IN PIRP Irp );
	virtual VOID OnStartWriteIo( IN PIRP Irp );

	// overridable dispatch handlers
	virtual NTSTATUS OnIrpMjCreate( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjCreateNamedPipe( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjClose( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjRead( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjWrite( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjQueryInformation( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjSetInformation( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjQueryEa( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjSetEa( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjFlushBuffers( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjQueryVolumeInformation( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjSetVolumeInformation( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjDirectoryControl( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjFileSystemControl( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjDeviceControl( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjInternalDeviceControl( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjShutdown( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjLockControl( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjCleanup( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjCreateMailslot( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjQuerySecurity( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjSetSecurity( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjQueryPower( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjSetPower( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjDeviceChange( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjQueryQuota( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjSetQuota( IN PIRP Irp );
	virtual NTSTATUS OnIrpMjDefault( IN PIRP Irp );

	virtual VOID GetDeviceDescription( OUT PDEVICE_DESCRIPTION DeviceDescription );

protected:
	// called from Unload() member.
	virtual VOID OnUnload( VOID );

	// called by CDeviceObject::Initialize(), which is called from
	// CDriverObject::Initialize() during driver initialization.
	// IMPORTANT:
	//		A class derived from CDeviceObject MUST call
	//		CInterruptObject::Initialize() for it's CInterruptObject member,
	//		if it exists.
	virtual NTSTATUS OnInitialize( IN PUNICODE_STRING RegistryPath );

    // called by Cancel().
	virtual VOID OnCancel( IN PIRP Irp );

	// calls IoStartPacket() with itself as the target device object.
	VOID StartPacket( IN PIRP Irp, IN PULONG Key = NULL, IN BOOLEAN bCancellable = TRUE );

	// calls IoInitializeTimer(). A driver's IoTimer routine is called once per
	// second after the driver enables the timer by calling IoStartTimer. The
	// driver can disable the timer by calling StopTimer and can re-enable it
	// again with StartTimer. Callers of IoInitializeTimer must be running at
	// IRQL PASSIVE_LEVEL. Should be called by derived class object in it's
	// OnInitialize() routine.
	NTSTATUS InitializeTimer( IN PIO_TIMER_ROUTINE TimerRoutine, IN PVOID Context );

	// Calls IoRaiseHardError().
	VOID RaiseHardError( IN PIRP Irp, IN PVPB Vpb );

	// Calls IoRequestDpc().
	VOID RequestDpc( IN PIRP Irp, IN PVOID Context );

	// Calls IoStartNextPacket().
	VOID StartNextPacket( IN BOOLEAN bCancellable = TRUE );

	// Calls IoStartNextPacketByKey().
	VOID StartNextPacket( IN BOOLEAN bCancellable, ULONG Key );

	// Calls IoStartTimer().
	VOID StartTimer( VOID );

	// Calls IoStopTimer().
	VOID StopTimer( VOID );

	// Calls IoUnregisterShutdownNotification().
	VOID UnregisterShutdownNotification( VOID );

	// Calls IoRegisterShutdownNotification().
	NTSTATUS RegisterShutdownNotification( VOID );

	// Calls IoCancelIrp(). IoCancelIrp() is bracketed by
	// IoAquireCancelSpinLock() and IoReleaseCancelSpinLock().
	BOOLEAN CancelIrp( IN PIRP Irp );

	BOOLEAN PciFindDevice(
		IN USHORT VendorId, 
		IN USHORT DeviceId, 
		OUT PULONG BusNumber, 
		OUT PPCI_SLOT_NUMBER PciSlotNumber, 
		OUT PPCI_COMMON_CONFIG PciCommonConfig );

	// Calls HalAssignSlotResources()
	NTSTATUS AssignSlotResources(
		IN PUNICODE_STRING RegistryPath, 
		IN PUNICODE_STRING DriverClassName, 
		IN INTERFACE_TYPE BusType,  
		IN ULONG BusNumber, 
		IN ULONG SlotNumber, 
		OUT PCM_RESOURCE_LIST* AllocatedResources );

	// Calls IoAssignResources().
	NTSTATUS AssignResources(
		IN PUNICODE_STRING RegistryPath, 
		IN PUNICODE_STRING DriverClassName, 
		IN PIO_RESOURCE_REQUIREMENTS_LIST RequestedResources, 
		OUT PCM_RESOURCE_LIST* AllocatedResources );

	NTSTATUS ReportResourceUsage(
		IN PUNICODE_STRING DriverClassName,
		IN PCM_RESOURCE_LIST DeviceList,
		IN ULONG DeviceListSize,
		IN BOOLEAN bOverRideConflict,
		OUT PBOOLEAN bConflictDetected );

private:
	// cancel routine.
	static VOID Cancel( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp );

protected:
	// members
	PDEVICE_OBJECT m_DeviceObject;
	CDriverObject* m_DriverObject;
	UNICODE_STRING m_Win32DeviceName;
	UNICODE_STRING m_RegistryPath;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline CDriverObject* CDeviceObject::GetDriver( void )
{
	return m_DriverObject;
}

inline NTSTATUS CDeviceObject::InitializeTimer( PIO_TIMER_ROUTINE TimerRoutine, PVOID Context )
{
	return IoInitializeTimer( *this, TimerRoutine, Context );
}

inline VOID CDeviceObject::StartTimer( void )
{
	IoStartTimer( m_DeviceObject );
	return;
}

inline VOID CDeviceObject::StopTimer( void )
{
	IoStopTimer( m_DeviceObject );
	return;
}

inline VOID CDeviceObject::RaiseHardError( PIRP Irp, PVPB Vpb )
{
	IoRaiseHardError( Irp, Vpb, m_DeviceObject );
	return;
}

inline VOID CDeviceObject::StartPacket(
	PIRP Irp,
	PULONG Key /* = NULL */,
	BOOLEAN bCancellable /* = TRUE*/ )
{
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	IoStartPacket( m_DeviceObject, Irp, Key, bCancellable ? Cancel : NULL );
	return;
}

inline VOID CDeviceObject::StartNextPacket( BOOLEAN bCancellable /* = TRUE */ )
{
	IoStartNextPacket( m_DeviceObject, bCancellable );
	return;
}

inline VOID CDeviceObject::StartNextPacket( BOOLEAN bCancellable, ULONG Key )
{
	IoStartNextPacketByKey( m_DeviceObject, bCancellable, Key );
	return;
}

inline NTSTATUS CDeviceObject::RegisterShutdownNotification( void )
{
	return IoRegisterShutdownNotification( m_DeviceObject );
}

inline VOID CDeviceObject::UnregisterShutdownNotification( void )
{
	IoUnregisterShutdownNotification( m_DeviceObject );
	return;
}

inline VOID CDeviceObject::GetDeviceDescription( OUT PDEVICE_DESCRIPTION DeviceDescription )
{
	RtlZeroMemory( DeviceDescription, sizeof DEVICE_DESCRIPTION );
	return;
}

inline VOID CDeviceObject::RequestDpc( PIRP Irp, PVOID Context )
{
	IoRequestDpc( m_DeviceObject, Irp, Context );
	return;
}

#endif device_h

/******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\device.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:30   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.20   11 Jun 1996 11:31:36   JDELLIOT
 * added support for all known Irps
 * added default support for uknown Irps
 * 
 *    Rev 1.19   31 May 1996 15:49:48   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 */
