////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\driver.cpv   1.0   24 Jun 1996 16:24:24   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:24  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	driver.cpp
// Abstract:	CDriverObject Implementation
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4705 )	// statement has no effect
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <idrivrpp.h>
#include "cppx.h"
#include "driver.h"

#define DEFAULT_CLASS_NAME L"Other"

////////////////////////////////////////////////////////////////////////////////
// static initialzation
//
CDriverObject* CDriverObject::s_DriverObject = NULL;

////////////////////////////////////////////////////////////////////////////////
// ctor/dtor
//
CDriverObject::CDriverObject( IN PCWSTR DriverClassName /* = NULL */ )
	:	m_DriverObject( NULL )
{
	// Set the class name for the driver
	//@@@ why not use our CUnicodeString class
	if( NULL != DriverClassName )
	{
		RtlInitUnicodeString( &m_ClassName, DriverClassName );
	}
	else
	{
		RtlInitUnicodeString( &m_ClassName, DEFAULT_CLASS_NAME );
	}

	ASSERT( NULL != s_DriverObject );
	s_DriverObject = this;
	return;
}

CDriverObject::~CDriverObject()
{
	s_DriverObject = NULL;
	return;
}

PVOID CDriverObject::operator new( IN size_t /* size */ )
{
	// Dynamic allocation not supported.
	return NULL;
}

NTSTATUS CDriverObject::DriverEntry(
	IN PDRIVER_OBJECT DriverObject,
	IN PUNICODE_STRING RegistryPath )
{
	// setup the bare minimum entry points
	s_DriverObject->SetDriverObject( DriverObject );
	DriverObject->DriverUnload = Unload;

	NTSTATUS Status;
	if( ! NT_SUCCESS( Status = s_DriverObject->Initialize( RegistryPath ) ) )
	{
		// initalization failed, unload the driver
		Unload( DriverObject );
	}

	return Status;
}

NTSTATUS CDriverObject::Initialize( IN PUNICODE_STRING RegistryPath )
{
	// create the devices for this driver
	NTSTATUS status;
	if( NT_SUCCESS( status = CreateDevices() ) )
	{
		// If we successfully create all of the devices, finish the
		// initialization of the driver object.
		if( NT_SUCCESS( status = InitDevices( RegistryPath ) ) )
		{
			status = OnInitialize( RegistryPath );

			// either by CreateDevices or OnInitialize, there should be at
			// least one device (though not neccessarily a CDeviceObject)
			ASSERT( NULL != m_DriverObject->DeviceObject );
		}
	}

	return status;
}

NTSTATUS CDriverObject::OnInitialize( IN PUNICODE_STRING /* RegistryPath */ )
{
	// setup the entry points
	m_DriverObject->DriverStartIo = StartIo;

	// Setup the dispatch table
	m_DriverObject->MajorFunction[ IRP_MJ_CLEANUP ] = IrpMjXxx;
	m_DriverObject->MajorFunction[ IRP_MJ_CLOSE ] = IrpMjXxx;
	m_DriverObject->MajorFunction[ IRP_MJ_CREATE ] = IrpMjXxx;
	m_DriverObject->MajorFunction[ IRP_MJ_DEVICE_CONTROL ] = IrpMjXxx;
	m_DriverObject->MajorFunction[ IRP_MJ_READ ] = IrpMjXxx;
	m_DriverObject->MajorFunction[ IRP_MJ_SHUTDOWN ] = IrpMjXxx;
	m_DriverObject->MajorFunction[ IRP_MJ_WRITE ] = IrpMjXxx;

//	m_DriverObject->MajorFunction[ IRP_MJ_FLUSH_BUFFERS ] = IrpMjXxx;
//	m_DriverObject->MajorFunction[ IRP_MJ_INTERNAL_DEVICE_CONTROL ] = IrpMjXxx;
//	m_DriverObject->MajorFunction[ IRP_MJ_QUERY_INFORMATION ] = IrpMjXxx;
//	m_DriverObject->MajorFunction[ IRP_MJ_SET_INFORMATION ] = IrpMjXxx;

	return STATUS_SUCCESS;
}

NTSTATUS CDriverObject::InitDevices( PUNICODE_STRING RegistryPath )
{
	// loop through list of devices (maintained by IO Manager)
	PDEVICE_OBJECT pDeviceObject = m_DriverObject->DeviceObject;
	while( NULL != pDeviceObject )
	{
		// get device object
		CDeviceObject* pDevice = (CDeviceObject*) pDeviceObject->DeviceExtension;
		ASSERT( NULL != pDevice );

		// call the most-derived's Initailize routine
		NTSTATUS status;
		if( ! NT_SUCCESS( (status = pDevice->Initialize( RegistryPath )) ) )
		{
			// Initialize failed... return (CDriverObject::Initialize will cleanup)
			return status;
		}

		// Register for shutdown notification for device
		IoRegisterShutdownNotification( pDeviceObject );

		// get the next device
		pDeviceObject = pDeviceObject->NextDevice;
	}
	
	return STATUS_SUCCESS;
}

VOID CDriverObject::Unload( IN PDRIVER_OBJECT /* DriverObject */ )
{
	ASSERT( NULL != s_DriverObject );
	s_DriverObject->OnUnload();
	s_DriverObject->UnloadDevices();

	cppShutdown();
	return;
}

VOID CDriverObject::OnUnload()
{
	return;
}

VOID CDriverObject::UnloadDevices()
{
	// loop through list of devices (maintained by IO Manager)
	PDEVICE_OBJECT pDeviceObject = m_DriverObject->DeviceObject;
	while( NULL != pDeviceObject )
	{
		// get device object
		CDeviceObject* pDevice = (CDeviceObject*) pDeviceObject->DeviceExtension;
		ASSERT( NULL != pDevice );

		// call the most-derived device object's Unload routine
		pDevice->Unload();

		// get the next device (before we delete this one)
		pDeviceObject = pDeviceObject->NextDevice;

		// delete the device object
		delete pDevice;
	}

	return;
}

VOID CDriverObject::StartIo( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp )
{
	ASSERT( NULL != s_DriverObject );
	s_DriverObject->OnStartIo( DeviceObject, Irp );
	return;
}

VOID CDriverObject::OnStartIo( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp )
{
	ASSERT_IRQL_EQ( DISPATCH_LEVEL );

	// get device object
	CDeviceObject* pObject = (CDeviceObject*) DeviceObject->DeviceExtension;
	ASSERT( NULL != pObject );

	// get stack location
	PIO_STACK_LOCATION CurrentStackLocation = IoGetCurrentIrpStackLocation( Irp );
	ASSERT( NULL != CurrentStackLocation );

	// dispatch to the CDeviceObject derived handlers
	switch( CurrentStackLocation->MajorFunction )
	{
	case IRP_MJ_READ:
		pObject->OnStartReadIo( Irp );
		break;

	case IRP_MJ_WRITE:
		pObject->OnStartWriteIo( Irp );
		break;

	default:
		pObject->OnStartIo( Irp );
		break;
	}

	return;
}

NTSTATUS CDriverObject::IrpMjXxx(
	IN PDEVICE_OBJECT DeviceObject,
	IN PIRP Irp )
{
	ASSERT( NULL != s_DriverObject );
	return s_DriverObject->OnIrpMjXxx( DeviceObject, Irp );
}

NTSTATUS CDriverObject::OnIrpMjXxx(
	IN PDEVICE_OBJECT DeviceObject, 
	IN PIRP Irp )
{
	// get device object
	CDeviceObject* pObject = (CDeviceObject*) DeviceObject->DeviceExtension;
	ASSERT( NULL != pObject );

	// get stack location
	PIO_STACK_LOCATION CurrentStackLocation = IoGetCurrentIrpStackLocation( Irp );
	ASSERT( NULL != CurrentStackLocation );

	// dispatch to the CDeviceObject derived handlers
	// @@@ we could add parameter "crackers" to these calls... (i.e. pass as
	// @@@ parameters the most commonly used members of the IO_STACK_LOCATION)
	switch( CurrentStackLocation->MajorFunction )
	{
	case IRP_MJ_CREATE:
		return pObject->OnIrpMjCreate( Irp );

	case IRP_MJ_CREATE_NAMED_PIPE:
		return pObject->OnIrpMjCreateNamedPipe( Irp );

	case IRP_MJ_CLOSE:
		return pObject->OnIrpMjClose( Irp );

	case IRP_MJ_READ:
		return pObject->OnIrpMjRead( Irp );

	case IRP_MJ_WRITE:
		return pObject->OnIrpMjWrite( Irp );

	case IRP_MJ_QUERY_INFORMATION:
		return pObject->OnIrpMjQueryInformation( Irp );

	case IRP_MJ_SET_INFORMATION:
		return pObject->OnIrpMjSetInformation( Irp );

	case IRP_MJ_QUERY_EA:
		return pObject->OnIrpMjQueryEa( Irp );

	case IRP_MJ_SET_EA:
		return pObject->OnIrpMjSetEa( Irp );

	case IRP_MJ_FLUSH_BUFFERS:
		return pObject->OnIrpMjFlushBuffers( Irp );

	case IRP_MJ_QUERY_VOLUME_INFORMATION:
		return pObject->OnIrpMjQueryVolumeInformation( Irp );

	case IRP_MJ_SET_VOLUME_INFORMATION:
		return pObject->OnIrpMjSetVolumeInformation( Irp );

	case IRP_MJ_DIRECTORY_CONTROL:
		return pObject->OnIrpMjDirectoryControl( Irp );

	case IRP_MJ_FILE_SYSTEM_CONTROL:
		return pObject->OnIrpMjFileSystemControl( Irp );

	case IRP_MJ_DEVICE_CONTROL:
		return pObject->OnIrpMjDeviceControl( Irp );

	case IRP_MJ_INTERNAL_DEVICE_CONTROL:
		return pObject->OnIrpMjInternalDeviceControl( Irp );

	case IRP_MJ_SHUTDOWN:
		return pObject->OnIrpMjShutdown( Irp );

	case IRP_MJ_LOCK_CONTROL:
		return pObject->OnIrpMjLockControl( Irp );

	case IRP_MJ_CLEANUP:
		return pObject->OnIrpMjCleanup( Irp );

	case IRP_MJ_CREATE_MAILSLOT:
		return pObject->OnIrpMjCreateMailslot( Irp );

	case IRP_MJ_QUERY_SECURITY:
		return pObject->OnIrpMjQuerySecurity( Irp );

	case IRP_MJ_SET_SECURITY:
		return pObject->OnIrpMjSetSecurity( Irp );

	case IRP_MJ_QUERY_POWER:
		return pObject->OnIrpMjQueryPower( Irp );

	case IRP_MJ_SET_POWER:
		return pObject->OnIrpMjSetPower( Irp );

	case IRP_MJ_DEVICE_CHANGE:
		return pObject->OnIrpMjDeviceChange( Irp );

	case IRP_MJ_QUERY_QUOTA:
		return pObject->OnIrpMjQueryQuota( Irp );

	case IRP_MJ_SET_QUOTA:
		return pObject->OnIrpMjSetQuota( Irp );

	default:
		return pObject->OnIrpMjDefault( Irp );
	}
}

NTSTATUS CDriverObject::ReportResourceUsage(
	IN PCM_RESOURCE_LIST DriverList, 
	IN ULONG DriverListSize, 
	IN BOOLEAN OverRideConflict, 
	OUT PBOOLEAN ConflictDetected )
{
	ASSERT( NULL != DriverList );
	ASSERT( NULL != ConflictDetected );

	return IoReportResourceUsage(
		&m_ClassName, 
		m_DriverObject, 
		DriverList, 
		DriverListSize, 
		NULL, 
		NULL, 
		0, 
		OverRideConflict,
		ConflictDetected );
}

NTSTATUS CDriverObject::AssignSlotResources(
	IN PUNICODE_STRING RegistryPath, 
	IN INTERFACE_TYPE BusType,  
	IN ULONG BusNumber, 
	IN ULONG SlotNumber, 
	OUT PCM_RESOURCE_LIST* AllocatedResources )
{
	ASSERT( NULL != RegistryPath );
	ASSERT( NULL != AllocatedResources );

	// If this CDeviceObject is an exclusive device object for this driver, NULL should be passed
	// as the DeviceObject argument to HalAssignSlotResources(), so that device-specific registry
	// management is not required.  If this is not an exclusive CDeviceObject object, then it is
	// necessary to create and update the registry path passed to the DriverEntry routine in order
	// to maintain device-specific resource data for each set of resources claimed under the driver-
	// specific path contained in the DriverEntry's RegistryPath.
	return HalAssignSlotResources( 
		RegistryPath, 
		&m_ClassName,
		m_DriverObject,
		NULL,
		BusType,
		BusNumber,
		SlotNumber,
		AllocatedResources );
}

NTSTATUS CDriverObject::AssignResources(
	IN PUNICODE_STRING RegistryPath, 
	IN PIO_RESOURCE_REQUIREMENTS_LIST RequestedResources, 
	OUT PCM_RESOURCE_LIST* AllocatedResources )
{
	ASSERT( NULL != RegistryPath );
	ASSERT( NULL != RequestedResources );
	ASSERT( NULL != AllocatedResources );

	return IoAssignResources( 
		RegistryPath, 
		&m_ClassName,
		m_DriverObject,
		NULL,
		RequestedResources,
		AllocatedResources );
}

VOID CDriverObject::RegisterDriverReinitialization(
	IN PDRIVER_REINITIALIZE DriverReinitializationRoutine, 
	IN PVOID Context )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != DriverReinitializationRoutine );	// @@@ is this correct?
	IoRegisterDriverReinitialization( *this, DriverReinitializationRoutine, Context );
	return;
}

extern "C" NTSTATUS DriverEntry(
	IN PDRIVER_OBJECT DriverObject, 
	IN PUNICODE_STRING RegistryPath )
{
	cppInit();

	return CDriverObject::DriverEntry( DriverObject, RegistryPath );
}

////////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\driver.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:24   JHUCKINS
// Initial revision.
// 
//    Rev 1.21   14 Jun 1996 08:49:54   JDELLIOT
// fixed bug that caused Unload to be called twice on failure conditions
// 
//    Rev 1.20   11 Jun 1996 11:30:02   JDELLIOT
// moved dispatch table and StartIo setup to OnInitialize
// moved IoRegisterShutdownNotification to InitDevices
// added support for all know Irps
// added default support for unknown Irps
// 
//    Rev 1.19   03 Jun 1996 11:43:32   JDELLIOT
// changed dispatch handlers to a single handler
// 
//    Rev 1.18   31 May 1996 15:49:54   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
//    Rev 1.17   25 Apr 1996 14:10:42   JHUCKINS
// Register devices for shutdown notifications via
// IoRegisterShutdownNotification() in ::Initialize()
// 
//    Rev 1.16   22 Apr 1996 10:16:16   JHUCKINS
// Moved driver entry-points from CDeviceObject to CDriverObject so that the
// CDriverObject can have an opportunity to filter dispatches to its 
// CDeviceObjects.
// 
//    Rev 1.15   04 Apr 1996 09:45:20   JHUCKINS
// Added IN and OUT macros to formal argument lists
// 
//    Rev 1.14   03 Apr 1996 16:44:40   JHUCKINS
// Use IoReportResourceUsage() instead of IoAssignResources OR HalAssignSlot
// Resources() to claim resources.
// 
//    Rev 1.13   02 Apr 1996 16:32:04   JHUCKINS
// checkpoint
// 
//    Rev 1.12   01 Apr 1996 22:07:20   JHUCKINS
// delete CDeviceObject in Unload() after calling CDeviceObject::Unload()
// 
//    Rev 1.11   27 Mar 1996 15:18:08   JHUCKINS
// Call CDriverObject::Unload() from DriverEntry() C routine if CDriverObject::
// DriverEntry() does not return STATUS_SUCCESS.
// 
//    Rev 1.10   27 Mar 1996 14:12:54   JDELLIOT
// added PVCS keywords and support for warning level 4
//
