////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\file.h_v   1.0   24 Jun 1996 16:24:34   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:34  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	file.h
// Author:		Jeff Huckins
// Abstract:	CFile Interface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
//		Intended as a simple wrapper class for the ZwXxx() file routines.
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef file_h
#define file_h

#include "debug.h"

class CFile
{
public:
	// ctor/dtor
	CFile();
	virtual ~CFile();

	// methods for top-level drivers
	NTSTATUS Create(
		IN ACCESS_MASK DesiredAccess,
		IN POBJECT_ATTRIBUTES ObjectAttributes,
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		IN PLARGE_INTEGER AllocationSize,
		IN ULONG FileAttributes,
		IN ULONG ShareAccess,
		IN ULONG CreateDisposition,
		IN ULONG CreateOptions,
		IN PVOID EaBuffer,
		IN ULONG EaLength );

	NTSTATUS Read( 
		IN HANDLE Event,
		IN PIO_APC_ROUTINE ApcRoutine,
		IN PVOID ApcContext,
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		OUT PVOID Buffer,
		IN ULONG Length,
		IN PLARGE_INTEGER ByteOffset = NULL,
		IN PULONG Key = NULL );

	NTSTATUS Write(
		IN HANDLE Event,
		IN PIO_APC_ROUTINE ApcRoutine,
		IN PVOID ApcContext,
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		IN PVOID Buffer,
		IN ULONG Length,
		IN PLARGE_INTEGER ByteOffset = NULL,
		IN PULONG Key = NULL );

	// methods for intermediate and device drivers
	NTSTATUS Create(
		IN ACCESS_MASK DesiredAccess,
		IN POBJECT_ATTRIBUTES ObjectAttributes,
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		IN ULONG FileAttributes,
		IN ULONG ShareAccess,
		IN ULONG CreateDisposition,
		IN ULONG CreateOptions );

	NTSTATUS Read(
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		OUT PVOID Buffer,
		IN ULONG Length,
		IN PLARGE_INTEGER = NULL,
		IN PULONG Key = NULL );

	NTSTATUS Write(
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		IN PVOID Buffer,
		IN ULONG Length,
		IN PLARGE_INTEGER ByteOffset = NULL,
		IN PULONG Key = NULL );

	// methods for all drivers
	NTSTATUS Close( VOID );

	NTSTATUS QueryInformation(
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		OUT PVOID Information,
		IN ULONG Length,
		IN FILE_INFORMATION_CLASS FileInformationClass = FileBasicInformation );

	NTSTATUS SetInformation(
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		IN PVOID Information,
		IN ULONG Length,
		IN FILE_INFORMATION_CLASS FileInformationClass = FileBasicInformation );

	// cast operators
	operator HANDLE();
	operator PHANDLE();

private:
	// members
	HANDLE m_Handle;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline NTSTATUS CFile::Create(
	IN ACCESS_MASK DesiredAccess,
	IN POBJECT_ATTRIBUTES ObjectAttributes,
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	IN PLARGE_INTEGER AllocationSize,
	IN ULONG FileAttributes,
	IN ULONG ShareAccess,
	IN ULONG CreateDisposition,
	IN ULONG CreateOptions,
	IN PVOID EaBuffer,
	IN ULONG EaLength )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL == m_Handle );
	ASSERT( NULL != ObjectAttributes );
	ASSERT( NULL != IoStatusBlock );

	return ZwCreateFile(
		&m_Handle,
		DesiredAccess,
		ObjectAttributes,
		IoStatusBlock,
		AllocationSize,
		FileAttributes,
		ShareAccess,
		CreateDisposition,
		CreateOptions,
		EaBuffer,
		EaLength );
}

inline NTSTATUS CFile::Create(
	IN ACCESS_MASK DesiredAccess,
	IN POBJECT_ATTRIBUTES ObjectAttributes,
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	IN ULONG FileAttributes,
	IN ULONG ShareAccess,
	IN ULONG CreateDisposition,
	IN ULONG CreateOptions )
{
	return Create(
		DesiredAccess,
		ObjectAttributes,
		IoStatusBlock,
		NULL,	// AllocationSize must be NULL for intermediate and device drivers
		FileAttributes,
		ShareAccess,
		CreateDisposition,
		CreateOptions,
		NULL,	// EaBuffer must be NULL for intermediate and device drivers
		0 ); 	// EaLenght must be 0 for intermediate and device drivers
}

inline NTSTATUS CFile::Read( 
	IN HANDLE Event,
	IN PIO_APC_ROUTINE ApcRoutine,
	IN PVOID ApcContext,
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	OUT PVOID Buffer,
	IN ULONG Length,
	IN PLARGE_INTEGER ByteOffset /* = NULL */,
	IN PULONG Key /* = NULL */ )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != m_Handle );
	ASSERT( NULL != IoStatusBlock );
	ASSERT( NULL != Buffer );

	return ZwReadFile(
		m_Handle,
		Event,
		ApcRoutine,
		ApcContext,
		IoStatusBlock,
		Buffer,
		Length,
		ByteOffset,
		Key );
}

inline NTSTATUS CFile::Read( 
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	OUT PVOID Buffer,
	IN ULONG Length,
	IN PLARGE_INTEGER ByteOffset /* = NULL */,
	IN PULONG Key /* = NULL */ )
{
	return Read(
		NULL,	// Event MUST be NULL for intermediate and device drivers
		NULL,	// ApcRoutine MUST be NULL for intermediate and device drivers
		NULL,	// ApcContext MUST be NULL for intermediate and device drivers
		IoStatusBlock,
		Buffer,
		Length,
		ByteOffset,
		Key );
}

inline NTSTATUS CFile::Write(
	IN HANDLE Event,
	IN PIO_APC_ROUTINE ApcRoutine,
	IN PVOID ApcContext,
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	IN PVOID Buffer,
	IN ULONG Length,
	IN PLARGE_INTEGER ByteOffset /* = NULL */,
	IN PULONG Key /* = NULL */ )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != m_Handle );
	ASSERT( NULL != IoStatusBlock );
	ASSERT( NULL != Buffer );

	return ZwWriteFile(
		m_Handle,
		Event,
		ApcRoutine,
		ApcContext,
		IoStatusBlock,
		Buffer,
		Length,
		ByteOffset,
		Key );
}

inline NTSTATUS CFile::Write(
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	IN PVOID Buffer,
	IN ULONG Length,
	IN PLARGE_INTEGER ByteOffset /* = NULL */,
	IN PULONG Key /* = NULL */ )
{
	return Write(
		NULL,	// Event MUST be NULL for intermediate and device drivers
		NULL,	// ApcRoutine MUST be NULL for intermediate and device drivers
		NULL,	// ApcContext MUST be NULL for intermediate and device drivers
		IoStatusBlock,
		Buffer,
		Length,
		ByteOffset,
		Key );
}

inline NTSTATUS CFile::QueryInformation(
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	OUT PVOID Information,
	IN ULONG Length,
	IN FILE_INFORMATION_CLASS FileInformationClass /* = FileBasicInformation */ )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != m_Handle );
	ASSERT( NULL != IoStatusBlock );
	ASSERT( NULL != Information );

	return ZwQueryInformationFile(
		m_Handle,
		IoStatusBlock,
		Information,
		Length,
		FileInformationClass );
}

inline NTSTATUS CFile::SetInformation(
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	IN PVOID Information,
	IN ULONG Length,
	IN FILE_INFORMATION_CLASS FileInformationClass /* = FileBasicInformation */ )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != m_Handle );
	ASSERT( NULL != IoStatusBlock );
	ASSERT( NULL != Information );

	return ZwSetInformationFile(
		m_Handle,
		IoStatusBlock,
		Information,
		Length,
		FileInformationClass );
}

inline CFile::operator HANDLE()
{
	return m_Handle;
}

inline CFile::operator PHANDLE()
{
	return &m_Handle;
}

#endif file_h

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\file.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:34   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.7   03 Jun 1996 07:47:10   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.6   31 May 1996 15:50:30   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.5   07 May 1996 10:52:12   JHUCKINS
 * Got rid of use of CLargeInteger
 * 
 *    Rev 1.4   03 May 1996 10:20:22   JHUCKINS
 * Fixed bug for Write() and Read() members where we were dereferencing a
 * potentially NULL formal argument
 * 
 *    Rev 1.3   03 May 1996 09:10:50   JHUCKINS
 * Added operator PHANDLE()
 * 
 *    Rev 1.2   30 Apr 1996 08:25:20   JHUCKINS
 * Added QueryInformation() and SetInformation()
 * 
 *    Rev 1.1   29 Apr 1996 13:52:20   JHUCKINS
 * fixed missing inline keyword on one of the constructors.
 * 
 *    Rev 1.0   27 Apr 1996 15:56:54   JHUCKINS
 * Initial revision.
 */
