////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\intrpt.h_v   1.0   24 Jun 1996 16:24:30   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:30  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	intrpt.h
// Author:		Jeff Huckins
// Abstract:	CInterruptObject Interface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef intrpt_h
#define intrpt_h

#include "debug.h"

// forward declarations
class CDeviceObject;

class CInterruptObject 
{
public:
	// ctor/dtor
	CInterruptObject( IN CDeviceObject* DeviceObject = NULL );
	virtual ~CInterruptObject();

	// Should be called from derived CDeviceObject::OnInitialize().
	NTSTATUS Initialize(
		IN PUNICODE_STRING RegistryPath, 
		IN INTERFACE_TYPE InterfaceType, 
		IN ULONG BusNumber, 
		IN ULONG BusInterruptLevel, 
		IN ULONG BusInterruptVector );

	// Registers a device driver's ISR to be called when its device interrupts
	// on any of a given set of processors. Calls IoConnectInterrupt().
	// If a non-null SyncSpinLock argument is passed in, Connect() will call
	// KeInitializeSpinLock() for the caller. Therefore, DO NOT pass in a
	// pre-initialized SpinLock!
	NTSTATUS Connect(
		IN PKSPIN_LOCK SpinLock, 
		IN KIRQL SynchronizeIrql, 
		IN KINTERRUPT_MODE InterruptMode, 
		IN BOOLEAN ShareVector, 
		IN BOOLEAN FloatingSave );

	// Calls IoDisconnectInterrupt().
	// IMPORTANT:
	//		A CDeviceObject derived class MUST call Disconnect() on it's
	//		CInterruptObject member in it's OnUnload() member.
	VOID Disconnect( VOID );

	// Synchronizes the execution of a given routine with that of the ISR
	// associated with the given interrupt object pointer.
	// Passes CInterruptObject class object's this pointer as ServiceContext
	// argument to KeSynchronizeExecution(). The OnSynchronizeExecution()
	// member will be called as a result of this call.
	// OnSynchronizeExecution() is where shared data access should occur for
	// the device object.
	BOOLEAN SynchronizeExecutionForRead( VOID );
	BOOLEAN SynchronizeExecutionForWrite( VOID );
	BOOLEAN SynchronizeExecution(
		IN PKSYNCHRONIZE_ROUTINE SynchronizeRoutine, 
		IN PVOID Context );

	// Get member.
	KIRQL GetIrql( VOID );

protected:
	// called from Initialize().
	virtual NTSTATUS OnInitialize( IN PUNICODE_STRING RegistryPath );

	// called from Isr() member.
	virtual BOOLEAN OnInterrupt( VOID );

private:
	// Entry point for the driver-supplied ISR declared as follows.
	static BOOLEAN Isr( IN PKINTERRUPT Interrupt, IN PVOID ServiceContext );

	// entry points for a caller-supplied SynchCritSection routines whose
	// execution is to be synchronized with the execution of the ISR associated
	// with the interrupt object(s).
	static BOOLEAN SynchronizeRoutineForRead( IN PVOID SynchronizedContext );
	static BOOLEAN SynchronizeRoutineForWrite( IN PVOID SynchronizedContext );

protected:
	// Mapped system vector returned from HalGetInterruptVector().
	ULONG m_Vector;

	// Set to the bus-relative IRQL. On return, this is reset to the DIRQL for
	// the mapped vector in call to HalGetInterruptVector().
	KIRQL m_Irql;

	// Set to the system-assigned affinity mask. This mask determines the set
	// of processors on which the device can interrupt in call to
	// HalGetInterruptVector().
	KAFFINITY m_Affinity;

	// Driver-supplied storage for a pointer to a set of interrupt objects,
	// which was obtained by calling HalGetInterruptVector. This pointer must
	// be passed in subsequent calls to KeSynchronizeExecution.
	PKINTERRUPT m_InterruptObject;

	CDeviceObject* m_DeviceObject;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline BOOLEAN CInterruptObject::SynchronizeExecution(
	IN PKSYNCHRONIZE_ROUTINE SynchronizeRoutine, 
	IN PVOID Context )
{
	ASSERT( NULL != SynchronizeRoutine );
	ASSERT_IRQL_LE( m_Irql );
	return KeSynchronizeExecution( m_InterruptObject, SynchronizeRoutine, Context );
}

inline BOOLEAN CInterruptObject::SynchronizeExecutionForRead( void )
{
	return SynchronizeExecution( SynchronizeRoutineForRead, m_DeviceObject );
}

inline BOOLEAN CInterruptObject::SynchronizeExecutionForWrite( void )
{
	return SynchronizeExecution( SynchronizeRoutineForWrite, m_DeviceObject );
}

inline KIRQL CInterruptObject::GetIrql( void )
{
	return m_Irql;
}

#endif intrpt_h

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\intrpt.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:30   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.9   03 Jun 1996 07:47:04   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.8   31 May 1996 15:50:02   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.7   29 Apr 1996 11:21:14   JHUCKINS
 * made data members protected instead of private
 * 
 *    Rev 1.6   04 Apr 1996 09:45:22   JHUCKINS
 * Added IN and OUT macros to formal argument lists
 * 
 *    Rev 1.5   27 Mar 1996 14:59:00   JHUCKINS
 * Removed OnSynchronizeExecution()
 * 
 *    Rev 1.4   27 Mar 1996 14:48:28   JHUCKINS
 * Added SynchronizeExecution() members for Read and Write
 * 
 *    Rev 1.3   27 Mar 1996 14:13:06   JDELLIOT
 * added PVCS keywords and support for warning level 4
 *
 */

