////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\linteger.h_v   1.0   24 Jun 1996 16:24:34   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:34  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	linteger.h
// Abstract:	CLargeInteger interface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef linteger_h
#define linteger_h

///////////////////////////////////////////////////////////////////////////////
// CLargeInteger interface
//
class CLargeInteger
{
public:
	// ctor
	CLargeInteger( void );
	CLargeInteger( IN const LARGE_INTEGER Value );
	CLargeInteger( IN const int Value );

	// conversion operators
	operator LARGE_INTEGER( void ) { return m_LargeInteger; }
	operator LARGE_INTEGER*( void ) { return &m_LargeInteger; }

	// assignment operator
	CLargeInteger& operator =( IN const CLargeInteger& Value );
	CLargeInteger& operator =( IN const int Value );

	// arithmetic operators
	CLargeInteger& operator +=( IN const CLargeInteger& Value );
	CLargeInteger& operator -=( IN const CLargeInteger& Value );

	// friend operators
	friend CLargeInteger operator +( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );
	friend CLargeInteger operator -( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );
	friend operator >( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );
	friend operator >=( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );
	friend operator ==( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );
	friend operator !=( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );
	friend operator <( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );
	friend operator <=( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue );

	// public members to make CLargeInteger operate just like a LARGE_INTEGER
	LONG& 	HighPart;
	ULONG& 	LowPart;

private:
	// members
	LARGE_INTEGER	m_LargeInteger;
};

///////////////////////////////////////////////////////////////////////////////
// CLargeInteger inline functions
//
inline CLargeInteger& CLargeInteger::operator =( IN const CLargeInteger& Value )
{
	LowPart = Value.LowPart;
	HighPart = Value.HighPart;
	return *this;
}

inline CLargeInteger& CLargeInteger::operator =( IN const int Value )
{
	LowPart = Value;
	HighPart = 0;
	return *this;
}

inline CLargeInteger& CLargeInteger::operator +=( IN const CLargeInteger& Value )
{
	m_LargeInteger = ::RtlLargeIntegerAdd( m_LargeInteger, Value.m_LargeInteger );
	return *this;
}

inline CLargeInteger& CLargeInteger::operator -=( IN const CLargeInteger& Value )
{
	m_LargeInteger = ::RtlLargeIntegerSubtract( m_LargeInteger, Value.m_LargeInteger );
	return *this;
}

///////////////////////////////////////////////////////////////////////////////
// global arithmetic operators
//
inline CLargeInteger operator +( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return CLargeInteger( ::RtlLargeIntegerAdd( lValue.m_LargeInteger, rValue.m_LargeInteger ) );
}

inline CLargeInteger operator -( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return CLargeInteger( ::RtlLargeIntegerSubtract( lValue.m_LargeInteger, rValue.m_LargeInteger ) );
}

///////////////////////////////////////////////////////////////////////////////
// relational operators
//
inline operator >( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return RtlLargeIntegerGreaterThan( lValue.m_LargeInteger, rValue.m_LargeInteger );
}

inline operator >=( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return RtlLargeIntegerGreaterThanOrEqualTo( lValue.m_LargeInteger, rValue.m_LargeInteger );
}

inline operator ==( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return RtlLargeIntegerEqualTo( lValue.m_LargeInteger, rValue.m_LargeInteger );
}

inline operator !=( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return RtlLargeIntegerNotEqualTo( lValue.m_LargeInteger, rValue.m_LargeInteger );
}

inline operator <( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return RtlLargeIntegerLessThan( lValue.m_LargeInteger, rValue.m_LargeInteger );
}

inline operator <=( IN const CLargeInteger& lValue, IN const CLargeInteger& rValue )
{
	return RtlLargeIntegerLessThanOrEqualTo( lValue.m_LargeInteger, rValue.m_LargeInteger );
}

#endif linteger_h

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\linteger.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:34   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.4   11 Jun 1996 11:30:00   JDELLIOT
 * cleanup of naming
 * 
 *    Rev 1.3   31 May 1996 15:50:26   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.2   13 May 1996 13:26:42   JDELLIOT
 * fixed addition and subtraction support
 * 
 *    Rev 1.1   25 Apr 1996 17:39:18   JDELLIOT
 * fixed msvc 4.1 compatability problems
 * 
 *    Rev 1.0   25 Apr 1996 14:07:36   JDELLIOT
 * Initial revision.
 * 
 */

