////////////////////////////////////////////////////////////////////////////////
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// Module Name:		pcidev.h
// Abstract:		CPciDevice Interface
// Environment:		Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef PCIDEV_H
#define PCIDEV_H

class CPciDevice : public CDeviceObject
{
public:
	CPciDevice( IN CDriverObject* DriverObject );
	virtual ~CPciDevice();

	ULONG GetBusNumber( VOID );

protected:
	virtual NTSTATUS OnInitialize( IN PUNICODE_STRING RegistryPath );
	virtual VOID OnUnload( void );

	NTSTATUS ConnectInterrupt( VOID );
	VOID DisconnectInterrupt( VOID );
	ULONG GetBusData( OUT ULONG& Data, IN ULONG Offset );
	ULONG SetBusData( IN ULONG& Data, IN ULONG Offset );
	ULONG GetBusInterruptData( OUT ULONG& InterruptData );
	ULONG GetBusMemoryData( OUT ULONG& MemoryData );

	// call CInterruptObject::SychronizeExecutionForXXX 
	BOOLEAN SynchronizeExecutionForRead( VOID );
	BOOLEAN SynchronizeExecutionForWrite( VOID );

private:
	NTSTATUS DoCheckedBuildKludge( IN PUNICODE_STRING RegistryPath );

protected:
	CInterruptObject	m_Interrupt;
	CDpcObject			m_Dpc;
	ULONG				m_ulBusNumber;
	void*				m_pBaseAddress;
	ULONG				m_ulBaseLength;
	ULONG				m_ulSlotNumber;
	
	// some useful definitions
	enum
	{
		vendor_id = 0x1234,			// Your PCI vendor id
		device_id = 0x5555,			// Your PCI device id
		offset_interrupt = 0x3c,	// PCI configuration space interrupt offset
		offset_memory = 0x10		// PCI configuration space base address offset
	};
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline ULONG CPciDevice::GetBusNumber( void )
{
	return m_ulBusNumber;
}

inline ULONG CPciDevice::GetBusData( OUT ULONG& Data, IN ULONG Offset )
{
	return HalGetBusDataByOffset(
		PCIConfiguration,
		m_ulBusNumber,
		m_ulSlotNumber,
		&Data,
		Offset,
		sizeof Data );
}

inline ULONG CPciDevice::SetBusData( OUT ULONG& Data, IN ULONG Offset )
{
	return HalGetBusDataByOffset(
		PCIConfiguration,
		m_ulBusNumber,
		m_ulSlotNumber,
		&Data,
		Offset,
		sizeof Data );
}

inline ULONG CPciDevice::GetBusInterruptData( IN ULONG& InterruptData )
{
	ULONG Result = GetBusData( InterruptData, offset_interrupt );
	InterruptData = InterruptData & 0xff;
	return Result;
}

inline ULONG CPciDevice::GetBusMemoryData( IN ULONG& MemoryData )
{
	return GetBusData( MemoryData, offset_memory );
}

inline BOOLEAN CPciDevice::SynchronizeExecutionForRead( VOID )
{
	return m_Interrupt.SynchronizeExecutionForRead();
}

inline BOOLEAN CPciDevice::SynchronizeExecutionForWrite( VOID )
{
	return m_Interrupt.SynchronizeExecutionForWrite();
}

#endif bertdev_h

