////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\uldevice.cpv   1.0   24 Jun 1996 16:24:28   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:28  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	uldevice.cpp
// Author:		Jeff Huckins
// Abstract:	CUpperDeviceObject Implementation
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <idrivrpp.h>
#include "uldevice.h"

////////////////////////////////////////////////////////////////////////////////
// ctor/dtor
CUpperDeviceObject::CUpperDeviceObject( IN CDriverObject* DriverObject )
	:	CDeviceObject( DriverObject ),
		m_AttachedDeviceObject( NULL ),
		m_FileObject( NULL )
{
	return;
}

CUpperDeviceObject::~CUpperDeviceObject()
{
	DetachDeviceObject();
	return;
}

////////////////////////////////////////////////////////////////////////////////
// attach/detach to/from another device
//
NTSTATUS CUpperDeviceObject::AttachDeviceObject(
	IN PUNICODE_STRING TargetDeviceName )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != TargetDeviceName );
	NTSTATUS Status = IoAttachDevice( *this, TargetDeviceName, &m_AttachedDeviceObject );
	ASSERT( NULL != m_AttachedDeviceObject );
	return Status;
}

NTSTATUS CUpperDeviceObject::GetDeviceObjectPointer(
	IN PUNICODE_STRING TargetDeviceName,
	IN ACCESS_MASK DesiredAccess /* = FILE_ALL_ACCESS */,
	BOOLEAN AdjustStackSize /* = TRUE */ )
{
	NTSTATUS Status = IoGetDeviceObjectPointer(
							TargetDeviceName,
							DesiredAccess,
							&m_FileObject,
							&m_AttachedDeviceObject );

	if( STATUS_SUCCESS == Status && AdjustStackSize )
	{
	#pragma warning( disable: 4244 )	// conversion from 'const int' to 'char', possible loss of data
		m_DeviceObject->StackSize = m_AttachedDeviceObject->StackSize + 1;
	#pragma warning( default: 4244 )	// conversion from 'const int' to 'char', possible loss of data
	}

	return Status;
}

NTSTATUS CUpperDeviceObject::AttachDeviceObject(
	IN PDEVICE_OBJECT TargetDeviceObject )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != TargetDeviceObject );
	NTSTATUS Status = IoAttachDeviceByPointer( *this, TargetDeviceObject );
	ASSERT( NULL != m_AttachedDeviceObject );
	return Status;
}

VOID CUpperDeviceObject::DetachDeviceObject( void )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	if( NULL != m_AttachedDeviceObject )
	{
		IoDetachDevice( m_AttachedDeviceObject );
		m_AttachedDeviceObject = NULL;
	}

	if( NULL != m_FileObject )
		ObDereferenceObject( m_FileObject );

	return;
}

PIRP CUpperDeviceObject::BuildDeviceIoControlRequest(
	IN ULONG IoControlCode,
	IN PVOID InputBuffer,
	IN ULONG InputBufferLength,
	IN PVOID OutputBuffer,
	IN ULONG OutputBufferLength,
	IN BOOLEAN bInternalDeviceIoControl,
	IN PKEVENT Event,
	OUT PIO_STATUS_BLOCK StatusBlock )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	ASSERT( NULL != Event );
	ASSERT( NULL != StatusBlock );

	return IoBuildDeviceIoControlRequest( 
		IoControlCode, 
		m_AttachedDeviceObject, 
		InputBuffer, 
		InputBufferLength, 
		OutputBuffer, 
		OutputBufferLength, 
		bInternalDeviceIoControl, 
		Event, 
		StatusBlock );
}

////////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\uldevice.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:28   JHUCKINS
// Initial revision.
// 
//    Rev 1.12   06 Jun 1996 16:22:16   JHUCKINS
// Added m_FileObject data member to dereference after calls to 
// GetDeviceObjectPointer()
// 
//    Rev 1.11   06 Jun 1996 15:57:26   JHUCKINS
// Added GetDeviceObjectPointer()
// 
//    Rev 1.10   03 Jun 1996 07:47:06   JHUCKINS
// Added Author to file header
// 
//    Rev 1.9   31 May 1996 15:50:12   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
//    Rev 1.8   16 May 1996 14:05:06   JHUCKINS
// Initialize m_AttachedDeviceObject member to NULL in constructor
// 
//    Rev 1.7   16 May 1996 13:49:32   JHUCKINS
// Resolve pure virtual OnInterrupt() from CDeviceObject
// 
//    Rev 1.6   16 May 1996 13:44:22   JHUCKINS
// Resolved pure virtual GetDeviceDescription() from CDeviceObject
// 
//    Rev 1.5   16 May 1996 13:22:24   JHUCKINS
// Added CDriverObject* argument to constructor
// 
//    Rev 1.4   08 Apr 1996 08:51:08   JHUCKINS
// Added IN and OUT for formal parameters
// 
//    Rev 1.3   03 Apr 1996 07:29:40   JHUCKINS
// include idrivrpp.h instead of ntdevpp.h so that global operator new and
// delete are defined for all
// 
//    Rev 1.2   27 Mar 1996 14:13:26   JDELLIOT
// added PVCS keywords and support for warning level 4
//

