////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\unicode.cpv   1.0   24 Jun 1996 16:24:28   JHUCKINS  $
//
// INTEL Corporation Proprietary Information
//
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
//
// Copyright (c) 1996 Intel Corporation. All rights reserved.
//
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:28  $
// $Author:   JHUCKINS  $
//
// Log at end of file.
//
// Module Name:		unicode.cpp
// Abstract:		CUnicodeString Implementation
// Environment:		Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
//
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4705 )	// statement has no effect
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <ntdevpp.h>
#include <mem.h>
#include "unicode.h"

////////////////////////////////////////////////////////////////////////////////
// ctor/dtor
//
CUnicodeString::CUnicodeString( IN PUNICODE_STRING String /* = NULL */ )
	:	m_PoolType( MaxPoolType )
{
	m_Str.Buffer = NULL;
	m_Str.Length = 0;
	m_Str.MaximumLength = 0;

	if( NULL != String )
	{
		Copy( String, MaxPoolType );
	}

	return;
}

CUnicodeString::CUnicodeString( IN PCWSTR String )
	:	m_PoolType( MaxPoolType )
{
	m_Str.Buffer = NULL;
	m_Str.Length = 0;
	m_Str.MaximumLength = 0;

	Init( String, MaxPoolType );
	return;
}

CUnicodeString::CUnicodeString( IN const CUnicodeString& String )
	:	m_PoolType( MaxPoolType )
{
	m_Str.Buffer = NULL;
	m_Str.Length = 0;
	m_Str.MaximumLength = 0;

	Copy( (const PUNICODE_STRING) &String.m_Str, MaxPoolType );
	return;
}

CUnicodeString::~CUnicodeString()
{
	// do we own the string and is there a string
	if( MaxPoolType != m_PoolType && NULL != m_Str.Buffer )
	{
		delete m_Str.Buffer;
	}

	return;
}

////////////////////////////////////////////////////////////////////////////////
// copies a unicode string to this CUnicodeString object, allocating new space
// if necessary (due to size or pool type reasons)
//
BOOLEAN CUnicodeString::Copy( 
	IN const PUNICODE_STRING String,
	IN POOL_TYPE PoolType /* = MaxPoolType */ )
{
	// validate parameters
	if( NULL == String )
	{
		TRACE( "CUnicodeString::Copy - Failed because String was NULL!" );
		return FALSE;
	}

	// if we have no current buffer then just Init it
	if( NULL == m_Str.Buffer )
	{
		return Init( String->Buffer, PoolType );
	}

	// if new pooltype is MaxPoolType
	if( MaxPoolType == PoolType )
	{
		// if we owned the previous buffer, delete it
		if( MaxPoolType != m_PoolType )
		{
			delete m_Str.Buffer;
			m_PoolType = MaxPoolType;
		}

		// store new string
		m_Str.Buffer = String->Buffer;
		m_Str.Length = String->Length;
		m_Str.MaximumLength = String->MaximumLength;
	}
	else
	{
		// will the new string fit and is it the same pool type
		if( m_Str.Length < String->Length || m_PoolType == PoolType )
		{
			// either we are changing pool types or there wasn't enough room
			m_PoolType = PoolType;

			// delete the buffer, if neccessary
			if( NULL != m_Str.Buffer && MaxPoolType != m_PoolType )
			{
				delete m_Str.Buffer;
			}

			// set the actual values
			m_Str.Length = 0;
			m_Str.MaximumLength = String->MaximumLength;
			m_Str.Buffer = NULL;

			// try to allocate a new buffer
			if( FALSE == Allocate() )
			{
				TRACE( "CUnicodeString::Copy - Failed on call to Allocate!" );
				return FALSE;
			}
		}

		RtlCopyUnicodeString( &m_Str, String );
	}

	return TRUE;
}

BOOLEAN CUnicodeString::Copy(
	IN PCWSTR String,
	IN POOL_TYPE PoolType /* = MaxPoolType */ )
{
	CUnicodeString UnicodeString( String );
	return Copy( UnicodeString, PoolType );
}

#pragma warning( disable: 4244 )	// 'initializing' : conversion from 'int' to 'unsigned short', possible loss of data
BOOLEAN CUnicodeString::Copy(
	IN PSTR String,
	IN POOL_TYPE PoolType /* = NonPagedPool */ )
{
	ANSI_STRING AnsiString;
	RtlInitAnsiString( &AnsiString, String );

	// Do the safe thing.
	m_PoolType = MaxPoolType == PoolType ? NonPagedPool : PoolType;
	m_Str.Length = USHORT( RtlAnsiStringToUnicodeSize( &AnsiString ) );
	m_Str.MaximumLength = m_Str.Length + sizeof( WCHAR );

	// Allocate the space and then convert the string to unicode
	if( !Allocate() )
		return FALSE;

	// We've just allocated the space for the unicode string, so don't
	// have RtlAnsiStringToUnicodeString() do it.
	return STATUS_SUCCESS == RtlAnsiStringToUnicodeString(
									&m_Str,
									&AnsiString,
									FALSE );	// Don't allocate the destination string
}
#pragma warning( default: 4244 )	// 'initializing' : conversion from 'int' to 'unsigned short', possible loss of data

////////////////////////////////////////////////////////////////////////////////
// initializes a CUnicodeString object to a wide-char string
//
BOOLEAN CUnicodeString::Init(
	IN PCWSTR String /* = NULL */,
	IN POOL_TYPE PoolType /* = MaxPoolType */ )
{
	// are we attempting to initialize a CUnincodeString object that has a string
	if( NULL != m_Str.Buffer )
	{
		TRACE( "CUnicodeString::Init - Failed because NULL != m_Str.Buffer!" );
		return FALSE;
	}

	// store pool type (MaxPoolType implies that we don't own the string)
	m_PoolType = PoolType;

	// is the string null
	if( NULL == String )
	{
	 	m_Str.Length = 0;
	 	m_Str.MaximumLength = 0;
		m_Str.Buffer = NULL;
	}
	else
	{
		// do we own the string
		if( MaxPoolType == m_PoolType )
		{
			// no, just init the unicode string and succeed
			RtlInitUnicodeString( &m_Str, String );
		}
		else
		{
			m_Str.Length = USHORT( wcslen( String ) * sizeof WCHAR );
			m_Str.MaximumLength = USHORT( m_Str.Length + sizeof WCHAR );
			if( FALSE == Allocate() )
			{
				return FALSE;
			}

			RtlCopyMemory( m_Str.Buffer, String, m_Str.MaximumLength );
		}
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// allocate storage for this string
//
BOOLEAN CUnicodeString::Allocate( void )
{
	if( MaxPoolType == m_PoolType )
	{
		TRACE( "CUnicodeString::Allocate - Failed because we don't not own the buffer!" );
		return FALSE;
	}

	if( NULL != m_Str.Buffer )
	{
		TRACE( "CUnicodeString::Allocate - Failed because we already have a buffer!" );
		return FALSE;
	}

	// allocate the buffer (remember Length and MaximumLength are in bytes not chars)
	if( NULL == (m_Str.Buffer = PWSTR( new( m_PoolType ) char[m_Str.MaximumLength] ) ) )
	{
		TRACE( "CUnicodeString::Allocate - Failed to allocate the buffer!" );
		return FALSE;
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// append a string to this string
//
BOOLEAN CUnicodeString::Append( IN const PUNICODE_STRING String )
{
	// validate the parameter
	if( NULL == String )
	{
		TRACE( "CUnicodeString::Append - Failed because NULL == String!" );
		return FALSE;
	}

	// do we own the buffer
	if( MaxPoolType == m_PoolType )
	{
		TRACE( "CUnicodeString::Append - Failed because we don't not own the buffer!" );
		return FALSE;
	}

	USHORT RequiredLength = USHORT( m_Str.Length + String->Length + sizeof WCHAR );
	if( m_Str.MaximumLength < RequiredLength )
	{
		if( FALSE == ReAllocate( RequiredLength ) )
		{
			TRACE( "CUnicodeString::Append - Failed on call to ReAllocate!" );
			return FALSE;
		}
	}
		
	return BOOLEAN( STATUS_SUCCESS == RtlAppendUnicodeStringToString( &m_Str, String ) );
}

////////////////////////////////////////////////////////////////////////////////
//  reallocate a string (if the object owns it)... RequiredLength is in bytes
//
BOOLEAN CUnicodeString::ReAllocate( IN USHORT RequiredLength )
{
	// do we own the buffer
	if( MaxPoolType == m_PoolType )
	{
		TRACE( "CUnicodeString::ReAllocate - Failed because we don't not own the buffer!" );
		return FALSE;
	}

	// is the new length 0
	if( 0 == RequiredLength )
	{
		// delete original buffer
		delete m_Str.Buffer;

		// set members
		m_Str.Buffer = NULL;
		m_Str.Length = 0;
		m_Str.MaximumLength = 0;

		return TRUE;
	}

	// attempt to allocate new buffer (remember Length and MaximumLength are in bytes)
	UNICODE_STRING String;
	if( NULL == (String.Buffer = PWSTR( new( m_PoolType ) char[ RequiredLength ] )) )
	{
		TRACE( "CUnicodeString::ReAllocate - Failed to allocate new buffer!" );
		return FALSE;
	}

	// copy the old string to the new string (truncating if necessary)
	RtlCopyUnicodeString( &String, &m_Str );

	// delete the old string's buffer
	delete m_Str.Buffer;

	// assign the new string
	m_Str = String;
	m_Str.MaximumLength = RequiredLength;

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// change pool types
//
BOOLEAN CUnicodeString::Convert( IN POOL_TYPE NewPoolType )
{
	// if the type is unchanged, we're done
	if( NewPoolType == m_PoolType )
	{
		return TRUE;
	}

	// we must own the buffer to convert it
	if( MaxPoolType == m_PoolType )
	{
		TRACE( "CUnicodeString::Convert - Failed because we don't not own the buffer!" );
		return FALSE;
	}

	// we must own the buffer to convert it
	if( MaxPoolType == NewPoolType )
	{
		TRACE( "CUnicodeString::Convert - Failed because caller is attempting to dissown a buffer!" );
		return FALSE;
	}

	// is there a buffer to convert
	if( NULL != m_Str.Buffer )
	{
		// allocate the new buffer of the correct pool type
		// (remember Length and MaximumLength are in bytes, not chars)
		PWSTR Buffer = PWSTR( new( NewPoolType ) char[ m_Str.MaximumLength ] );
		if( NULL == Buffer )
		{
			TRACE( "CUnicodeString::Convert - Failed to allocate new buffer!" );
			return FALSE;
		}

		// Zero it, copy it and free the old buffer
		RtlZeroMemory( Buffer, m_Str.MaximumLength );
		RtlCopyMemory( Buffer, m_Str.Buffer, m_Str.Length );
		m_Str.Buffer[ m_Str.Length / sizeof WCHAR ] = UNICODE_NULL;
		delete m_Str.Buffer;
	}

	// store the new pool type
	m_PoolType = NewPoolType;

	return TRUE;
}

USHORT GetStringLength( IN PSTR String )
{
	USHORT Length = 0;
	PCHAR p = String;

	while( *p++ )
		Length++;

	return Length;
}


////////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\unicode.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:28   JHUCKINS
// Initial revision.
// 
//    Rev 1.13   18 Jun 1996 16:23:24   JHUCKINS
// Use RtlInitAnsiString instead of initializing it ourselves in ::Copy()
// for PSTR 
// 
//    Rev 1.12   04 Jun 1996 14:19:00   JHUCKINS
// Added Copy() for Ansi string
// 
//    Rev 1.11   03 Jun 1996 07:47:08   JHUCKINS
// Added Author to file header
// 
//    Rev 1.10   02 Jun 1996 15:58:50   JHUCKINS
// Fix bug in Allocate() where it returned NULL instead of TRUE
// 
//    Rev 1.9   31 May 1996 16:53:00   JDELLIOT
// fixed memory leak in Copy
// 
//    Rev 1.8   31 May 1996 16:35:16   JHUCKINS
// checkpoint
// 
//    Rev 1.7   31 May 1996 15:50:22   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
