////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\unicode.h_v   1.0   24 Jun 1996 16:24:32   JHUCKINS  $
//
// INTEL Corporation Proprietary Information
//
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
//
// Copyright (c) 1996 Intel Corporation. All rights reserved.
//
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:32  $
// $Author:   JHUCKINS  $
//
// Log at end of file.
//
// Module Name:		unicode.h
// Abstract:		CUnicodeString Interface
// Environment:		Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
//
////////////////////////////////////////////////////////////////////////////////

#ifndef unicode_h
#define unicode_h

#include "debug.h"

class CUnicodeString
{
public:
	// ctor/dtor
	CUnicodeString( IN PUNICODE_STRING String = NULL );
	CUnicodeString( IN PCWSTR String );
	CUnicodeString( IN const CUnicodeString& String );
	virtual ~CUnicodeString();

	// operations
	BOOLEAN IsValid( void ) const;
	BOOLEAN Init( IN PCWSTR String = NULL, IN POOL_TYPE PoolType = MaxPoolType );
	BOOLEAN Copy( IN PUNICODE_STRING const String, IN POOL_TYPE PoolType = MaxPoolType );
	BOOLEAN Copy( IN PCWSTR String, IN POOL_TYPE PoolType = MaxPoolType );
	BOOLEAN Copy( IN PSTR String, IN POOL_TYPE PoolType = NonPagedPool );
	BOOLEAN Append( IN const PUNICODE_STRING String );
	BOOLEAN Append( IN const CUnicodeString& String );
	BOOLEAN Convert( IN POOL_TYPE PoolType );
	USHORT GetLength( VOID );
	USHORT GetMaximumLength( VOID );

	// operators
	CUnicodeString& operator +=( const CUnicodeString& String );
	CUnicodeString& operator =( const CUnicodeString& String );
	operator PUNICODE_STRING();

private:
	// methods
	BOOLEAN Allocate( void );
	BOOLEAN ReAllocate( IN USHORT RequiredLength );

	// members
	UNICODE_STRING	m_Str;
	POOL_TYPE		m_PoolType;
};

USHORT GetStringLength( IN PSTR String );

///////////////////////////////////////////////////////
// inlines
//
inline BOOLEAN CUnicodeString::IsValid( void ) const
{
	return BOOLEAN( NULL != m_Str.Buffer );
}

inline BOOLEAN CUnicodeString::Append( IN const CUnicodeString& String )
{
	return Append( (const PUNICODE_STRING) &String.m_Str );
}

inline CUnicodeString& CUnicodeString::operator +=( const CUnicodeString& String )
{
	Append( String );
	return *this;
}

inline CUnicodeString& CUnicodeString::operator =( const CUnicodeString& String )
{
	Copy( (const PUNICODE_STRING) &String.m_Str, MaxPoolType );
	return *this;
}

inline CUnicodeString::operator PUNICODE_STRING()
{
	return &m_Str;
}

inline USHORT CUnicodeString::GetLength( VOID )
{
	return m_Str.Length;
}

inline USHORT CUnicodeString::GetMaximumLength( VOID )
{
	return m_Str.MaximumLength;
}

#endif unicode_h

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\unicode.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:32   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.8   04 Jun 1996 14:22:28   JHUCKINS
 * Default PoolType for Copy() of Ansi String changed to NonPagedPool
 * 
 *    Rev 1.7   04 Jun 1996 14:18:34   JHUCKINS
 * Added Copy() for Ansi string
 * 
 *    Rev 1.6   03 Jun 1996 07:47:08   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.5   31 May 1996 15:50:20   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 *
 */

